/**
 * Sales Archiving Web Service
 * -----------------------
 *
 * This piece of software is part of the Sales Archiving Web Service example.
 *
 * Simply speaking, the aim of this software is to deploy, on JOnAS 4, a web
 * service to which sales data can be sent. That service would then store all
 * received data in a database using EJB3s.
 *
 *
 * If this software works, it has been written by Savas Ali TOKMEN, who has a
 * web site on the http://ali.tokmen.com/ address. If it doesn't work and
 * you're experiencing problems with it, I don't know who wrote it.
 *
 * Created the 31st of March 2007
 */
package SalesArchivingService;

import java.util.logging.Logger;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

import javax.persistence.Query;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

/**
 * This is the actual implementation of the SalesArchivingBeanRemote interface.
 */
@Stateless
@Remote(SalesArchivingBeanRemote.class)
public class SalesArchivingBean implements SalesArchivingBeanRemote {
	@PersistenceContext
	private EntityManager entityManager = null;

	/**
	 * @see SalesArchivingBeanRemote#saveSalesData
	 */
	@TransactionAttribute(TransactionAttributeType.REQUIRED)
	public void saveSalesData(SalesItem item, SalesData[] datum) throws Exception {
		// Try to find the item, if not present create it
		SalesItemBean itemBean = null;
		{
			Query itemQuery = entityManager.createQuery("select object(b) from SalesItemBean b where b.name = :name");
			try {
				itemQuery.setParameter("name", item.name);
				itemBean = (SalesItemBean) itemQuery.getSingleResult();
			} catch (Exception e) {}

			if(itemBean == null) {
				itemBean = new SalesItemBean();
				itemBean.setName(item.name);
				entityManager.persist(itemBean);
				Logger.global.info("Will insert sales item bean: " + itemBean);
			}
		}

		// Add all sales data. Note that we don't need to add each data to the
		// item's "sales" list, as the the entity manager will fetch those and
		// automagically create the associated list as needed
		if(datum != null) {
			for( int i=0 ; i<datum.length ; i++ ) {
				SalesDataBean salesDataBean = new SalesDataBean();
				salesDataBean.setDate(datum[i].date);
				salesDataBean.setPrice(datum[i].price);
				salesDataBean.setItem(itemBean);
				entityManager.persist(salesDataBean);
				Logger.global.info("    Will insert sales data bean: " + salesDataBean);
			}
		}
	}
}
